// ********************************************************************
// ULOG 2-KANAL UART-DATENLOGGER
// Version 1.0 / Stand: 20.10.2010
// Autor: Matthias Kahnt
// AVR-Version / GGC 4.3.2
// Mikrocontroller: ATmega1284P
// Frequenz: 18,432MHz
// LCD DIP204-4 4x20
// USB-Stick Interface (optional)
// ********************************************************************
// Die Programmfunktionen drfen fr private Nutzung verwendet und
// verndert werden. Fr gewerbliche Nutzung ist die Zustimmung
// des Autors erforderlich.
// Kontakt: Matthias Kahnt, e-mail: az51@gmx.net
// ********************************************************************
// Die Nutzung erfolgt auf eigene Gefahr.
// Fr eventuell entstehende Schden wird keine Haftung bernommen.
// ********************************************************************
//
//************************************************************************
// AVR/GCC LIBS
//************************************************************************
#include "defines.h"            // Projektdefinitionen
#include <avr/io.h>             // IO-Grundfunktionen
#include <avr/interrupt.h>      // Interrupt-Funktionen
#include <avr/pgmspace.h>       // PROGMEM-Funktionen
#include <avr/eeprom.h>         // EEPROM-Funktionen
#include <util/delay.h>         // Pausenfunktionen
#include <string.h>             // Stringfunktion
#include <stdio.h>              // Standardfunktion
#include <stdlib.h>             // Standardbibliothek

#include "spi_avr.h"            // SPI-Funktionen
#include "dip204spi_avr.h"      // LCD-Funktionen
#include "usart_avr.h"          // UART-Funktionen
#include "vnc1l_avr_spi_fifo.h" // VNC1L-Funktionen

//************************************************************************
// Variablenbereich
//************************************************************************
struct awd_struktur awd;            // Variablen der Einstellungen
struct awd_struktur awd_ee EEMEM;   // dto., nochmal im EEPROM

// Einstellparameter
struct parameter_struktur parameter[] = {
  { /*  0  */  100,     6,     13,    0,   1,    1,   1,  "PIN3-BAUDRATE",       &awd.uart0_baudrate    },
  { /*  1  */  101,     3,      4,    0,   1,    2,   1,  "PIN3-DATENBITS",      &awd.uart0_datenbits   },
  { /*  2  */  102,     0,      2,    0,   1,    3,   1,  "PIN3-PARIT\x5BT",     &awd.uart0_parity      },
  { /*  3  */  110,     6,     13,    0,   1,    1,   1,  "PIN2-BAUDRATE",       &awd.uart1_baudrate    },
  { /*  4  */  111,     3,      4,    0,   1,    2,   1,  "PIN2-DATENBITS",      &awd.uart1_datenbits   },
  { /*  5  */  112,     0,      2,    0,   1,    3,   1,  "PIN2-PARIT\x5BT",     &awd.uart1_parity      },
  { /*  6  */  120,     1,      1,    0,   1,    4,   1,  "PIN2/3-POLARIT\x5BT", &awd.uart01_pol        },
  { /*  7  */  130,     0,      5,    0,   1,    5,   1,  "UART LOG-MODE",       &awd.uart_log_mode     },
  { /*  8  */  131,     0,      1,    0,   1,    6,   1,  "UART APP-MODE",       &awd.uart_app_mode     },
  { /*  9  */  140,     0,      1,    0,   1,    7,   1,  "USB-STICK FORMAT",    &awd.stick_dateiformat },
  { /* 10  */  141,     0,      1,    0,   1,    8,   1,  "USB-Stick SPEED",     &awd.stick_speed       },
  { /* 11  */  150,     2,      2,    0,   1,    9,   1,  "LCD BELEUCHTUNG",     &awd.lcd_hgbl          },
  { /* 12  */  255,     1,      0,    0,   0,   10,   0,  "FIRMWARE-VERSION",    &awd.software_version  }
};
#define PARA_MAX 12         /* Parameter-Anzahl (0 bis ... [Anzahl - 1]) */

union log_union logbuch;    // Speicher der empfangenen Datenbyte

struct lcd_struc lcd;       // LCD Variablen
struct log_struc logset;    // Logging Variablen
struct vnc_struc vnc;       // VNC Variablen 

volatile unsigned char ledimage         = 0;    // LED-Zustnde fr das Schieberegister
volatile unsigned char blink            = 1;    // LED Blinkstatus
volatile unsigned int  int_delay        = 0;    // LED Blinkrate
volatile unsigned long time_ms          = 0;    // Timecode [ms]
volatile unsigned char time_reset       = 0;    // Timecode Resetflag
volatile unsigned int  hgl_counter      = 0;    // LCD-Hintergrundbeleuchtung [ms] bis Abschaltung
volatile unsigned int  timeout_counter  = 0;    // System Timeout [ms]
volatile unsigned char taste_last       = 5;    // zuletzt gedrckte Taste (5=Timeout)
volatile unsigned int  taste_repeater   = 0;    // Zeit im [ms] bis zur autmomatische Tastenwiederholung
volatile unsigned char temp_char        = 0;    // Universelle Verwendung

// ********************************************************************
// Prfroutine, ob gerade eine Taste gedrckt ist, ohne warten
// Rckgabe: Tastencode der gedrckten Taste
//           0 = keine Taste gedrckt
// keine Auswertung, wenn mehrere Tasten gleichzeitig gedrckt sind
// ********************************************************************
unsigned char keypressed(void) {
unsigned char key_code = 0;
  if (!TASTE_ESC)  key_code = ESC;      // Taste  ESC gedrckt
  if (!TASTE_SETM) key_code = SETM;     // Taste SET- gedrckt
  if (!TASTE_SETP) key_code = SETP;     // Taste SET+ gedrckt
  if (!TASTE_ET)   key_code = ET;       // Taste   ET gedrckt
  return (key_code);
}

// ********************************************************************
// Tastatur-Abfrage - mit Tastenwiederholung
// Warten auf Tastendruck und Rckgabe des Codes der gedrckten Taste
// Abbruch:  System-Timeout
// Rckgabe: Tastencode der gedrckten Taste
//           5 = Timeout
// keine Auswertung, wenn mehrere Tasten gleichzeitig gedrckt sind
// ********************************************************************

unsigned char get_taste(unsigned char repeat_fkt) {
unsigned char key_code = 0;
  if (repeat_fkt) {
    // Repeatfunktion
    if (taste_last) {
      // letzter Tastendruck
      while(1) {
        key_code = keypressed();            // aktueller Tastendruck
        if (key_code != taste_last) break;  // Neue Taste oder losgelassen
        if (!taste_repeater) break;         // Wiederholungspause abgelaufen
      }
    }
    if (!taste_repeater) {
      // Wiederholzeit abgelaufen, aber die letzte Taste ist immernoch gedrckt
      taste_repeater = TASTEN_REP_ZT; // Zeit im [ms] fr autmomatische Tastenwiederholung neu setzen
    } else {
      // Taste inzwischen losgelassen oder neue Taste
      timeout_counter   = MENUE_TIMEOUT;
      while (1) {
        while(keypressed());  // Warten falls Taste noch nicht losgelassen
        _delay_ms(50);
        // Warten auf Tastendruck
        while(!keypressed() && timeout_counter);
        // Eine Taste wurde gedrckt, oder Timeout eingetreten
        _delay_ms(50); // kurz warten, falls es nur prellen war, dann Taste ermitteln
        if (!TASTE_ESC)       key_code = ESC;    // Taste  ESC  gedrckt
        if (!TASTE_SETM)      key_code = SETM;   // Taste  SET- gedrckt
        if (!TASTE_SETP)      key_code = SETP;   // Taste  SET+ gedrckt
        if (!TASTE_ET)        key_code = ET;     // Taste  ET   gedrckt
        if (!timeout_counter) key_code = 5;      // Timeout eingetreten
        if (key_code) break;
      }
      taste_repeater = TASTEN_REP_BEG;    // Zeit in [ms] fr den Beginn der autmomatische Tastenwiederholung
      taste_last = key_code;              // zuletzt gedrckte Taste merken
    }
  } else {
    // Ohne Repeatfunktion
    timeout_counter   = MENUE_TIMEOUT;
    while (1) {
      while(keypressed());  // Warten falls Taste noch nicht losgelassen
      _delay_ms(50);
      // Warten auf Tastendruck
      while(!keypressed() && timeout_counter);
      // Eine Taste wurde gedrckt, oder Timeout eingetreten
      _delay_ms(50); // kurz warten, falls es nur prellen war, dann Taste ermitteln
      if (!TASTE_ESC)       key_code = ESC;    // Taste  ESC  gedrckt
      if (!TASTE_SETM)      key_code = SETM;   // Taste  SET- gedrckt
      if (!TASTE_SETP)      key_code = SETP;   // Taste  SET+ gedrckt
      if (!TASTE_ET)        key_code = ET;     // Taste  ET   gedrckt
      if (!timeout_counter) key_code = 5;      // Timeout eingetreten
      if (key_code) break;
    }
    taste_repeater = TASTEN_REP_BEG;    // Zeit in [ms] fr den Beginn der autmomatische Tastenwiederholung
    taste_last = key_code;              // zuletzt gedrckte Taste merken
  }
  return (taste_last);
}

// ********************************************************************
// UART-Logging
// Die empfangenen Bytes werden gespeichert und angezeigt. Die Anzeige
// erfolgt gebuffert (nachlaufend).
// Abbruch mit Taste <ESC> oder bei vollem Datenbuffer
// ********************************************************************
void logging(void) {
unsigned char taste;
unsigned char maske             = 0;
unsigned char maske_change      = 1;
unsigned int  index_alt         = 0;
unsigned char lcd_temp          = 0;
unsigned char lcd_x             = 0;
unsigned char temp_rxd_pin2     = 0;
unsigned char temp_rxd_pin3     = 0;
unsigned char temp_rxd_pin2_c   = 0;
unsigned char temp_rxd_pin3_c   = 0;
unsigned char temp_status       = 0;
unsigned int  temp_logset_index = 0;

// Anzeige-Texte
char *log_mode_TXT[] = {
   /*  0 */ "<PIN2-PIN3-ST-Time>",
   /*  1 */ "<PIN2-PIN3-ST>",
   /*  2 */ "<PIN2-PIN3-Time>",
   /*  3 */ "<PIN2-PIN3>",
   /*  4 */ "<PIN2>",
   /*  5 */ "<PIN3>" 
};

  clear_lcd();
  sprintf(&lcd.buffer[0],"Mode: %1d %s -bereit-",awd.uart_log_mode,awd.uart_app_mode ? "APP" : "NEW");
  printf_lcd(0,0,lcd.buffer,1);
  printf_lcd(0,1,log_mode_TXT[awd.uart_log_mode],1);
  printf_lcd(0,3," \x18",1);

  if (!awd.uart_app_mode) {
    // New-Mode, 
    memset(&logbuch,0,sizeof(logbuch));     // Speicher lschen
    logset.index = 0;
  } else {
    // Append-Mode
    temp_logset_index = logset.index;       // Fortfhren, ohne Speicher lschen
  }
  // Log-Funktion freigeben und auf erstes Byte warten
  logset.enable = 1;
  if (!awd.uart_app_mode) {
    printf_lcd(0,2,"System wartet...",1);
  } else {
    printf_lcd(0,2,"Append-Mode...",1);
    _delay_ms(3000);
  }
  // Logging
  while(logset.enable) {
    if (temp_logset_index < logset.index) temp_logset_index++;
    if (!awd.uart_app_mode) {
      // New-Mode
      if (!temp_logset_index) {
        time_reset      = 1;      // Timecode zurcksetzen
      }
    }
    if (temp_logset_index) {
      // mind. 1 Byte empfangen oder im Speicher
      // Auswertung/Anzeige der Daten beginnen
      switch (awd.uart_log_mode) {
        case 0: // Log-Mode 0
                temp_rxd_pin2   = logbuch.mode0[temp_logset_index - 1].rxd_pin2;
                temp_rxd_pin3   = logbuch.mode0[temp_logset_index - 1].rxd_pin3;
                temp_status     = logbuch.mode0[temp_logset_index - 1].status;
                break;
        case 1: // Log-Mode 1
                temp_rxd_pin2   = logbuch.mode1[temp_logset_index - 1].rxd_pin2;
                temp_rxd_pin3   = logbuch.mode1[temp_logset_index - 1].rxd_pin3;
                temp_status     = logbuch.mode1[temp_logset_index - 1].status;
                break;
        case 2: // Log-Mode 2
                temp_rxd_pin2   = logbuch.mode2[temp_logset_index - 1].rxd_pin2;
                temp_rxd_pin3   = logbuch.mode2[temp_logset_index - 1].rxd_pin3;
                if (temp_rxd_pin3) temp_status |= 0x01;        // Kennung fr Pin3 gltig
                if (temp_rxd_pin2) temp_status |= 0x02;        // Kennung fr Pin2 gltig
                break;
        case 3: // Log-Mode 3
                temp_rxd_pin2   = logbuch.mode3[temp_logset_index - 1].rxd_pin2;
                temp_rxd_pin3   = logbuch.mode3[temp_logset_index - 1].rxd_pin3;
                if (temp_rxd_pin3) temp_status |= 0x01;        // Kennung fr Pin3 gltig
                if (temp_rxd_pin2) temp_status |= 0x02;        // Kennung fr Pin2 gltig
                break;
        case 4: // Log-Mode 4
                temp_rxd_pin2   = logbuch.mode4[temp_logset_index - 1].rxd_pin2;
                temp_rxd_pin3   = 0;
                temp_status |= 0x02;
                break;
        case 5: // Log-Mode 5
                temp_rxd_pin2   = 0;
                temp_rxd_pin3   = logbuch.mode5[temp_logset_index - 1].rxd_pin3;
                temp_status |= 0x01;
                break;
      }
      // ASCII Einschrnkungen
      if ((temp_rxd_pin2 < 0x20) || (temp_rxd_pin2 > 0x7F)) temp_rxd_pin2_c = 0xDD; else temp_rxd_pin2_c = temp_rxd_pin2;
      if ((temp_rxd_pin3 < 0x20) || (temp_rxd_pin3 > 0x7F)) temp_rxd_pin3_c = 0xDD; else temp_rxd_pin3_c = temp_rxd_pin3;
      // Anzeige der Daten, je nach Log-Mode
      switch (maske) {
        case 0: // Maske 1 (F3)
                if (awd.uart_log_mode < 2) {
                  sprintf(&lcd.buffer[0],"Mode %1d: %lds S:%02X",awd.uart_log_mode,time_ms/1000,temp_status);
                } else {
                  sprintf(&lcd.buffer[0],"Mode %1d: %lds",awd.uart_log_mode,time_ms/1000);
                }
                printf_lcd(0,0,lcd.buffer,1);
                
                sprintf(&lcd.buffer[0],"Buffer: %d/%d",temp_logset_index,logset.max);
                printf_lcd(0,1,lcd.buffer,1);
                
                switch(awd.uart_log_mode) {
                  case 0: // Log-Mode 0
                  case 1: // Log-Mode 1
                          if (temp_status & 0x02) {
                            sprintf(&lcd.buffer[0],"PIN2:%02X\xDF%c ",temp_rxd_pin2,temp_rxd_pin2_c);
                          } else {
                            sprintf(&lcd.buffer[0],"PIN2:-- - ");
                          }
                          printf_lcd(0,2,lcd.buffer,0);
                          if (temp_status & 0x01) {
                            sprintf(&lcd.buffer[0],"PIN3:%02X\xDF%c ",temp_rxd_pin3,temp_rxd_pin3_c);
                          } else {
                            sprintf(&lcd.buffer[0],"PIN3:-- - ");
                          }
                          printf_lcd(10,2,lcd.buffer,0);
                          break;
                  case 2: // Log-Mode 2
                  case 3: // Log-Mode 3
                          sprintf(&lcd.buffer[0],"PIN2:%02X\xDF%c ",temp_rxd_pin2,temp_rxd_pin2_c);
                          printf_lcd(0,2,lcd.buffer,0);
                          sprintf(&lcd.buffer[0],"PIN3:%02X\xDF%c ",temp_rxd_pin3,temp_rxd_pin3_c);
                          printf_lcd(10,2,lcd.buffer,0);
                          break;
                  case 4: // Log-Mode 4
                          sprintf(&lcd.buffer[0],"PIN2:%02X\xDF%c ",temp_rxd_pin2,temp_rxd_pin2_c);
                          printf_lcd(0,2,lcd.buffer,0);
                          sprintf(&lcd.buffer[0],"PIN3:-- - ");
                          printf_lcd(10,2,lcd.buffer,0);
                          break;
                  case 5: // Log-Mode 5
                          sprintf(&lcd.buffer[0],"PIN2:-- - ");
                          printf_lcd(0,2,lcd.buffer,0);
                          sprintf(&lcd.buffer[0],"PIN3:%02X\xDF%c ",temp_rxd_pin3,temp_rxd_pin3_c);
                          printf_lcd(10,2,lcd.buffer,0);
                          break;
                }
                printf_lcd(0,3," \x18     F1   F2   F3",1);
                break;
        case 1: // Maske 2 (F1)
                if (awd.uart_log_mode < 2) {
                  sprintf(&lcd.buffer[0],"Mode %1d: %lds S:%02X",awd.uart_log_mode,time_ms/1000,temp_status);
                } else {
                  sprintf(&lcd.buffer[0],"Mode %1d: %lds",awd.uart_log_mode,time_ms/1000);
                }
                printf_lcd(0,0,lcd.buffer,0);
                if (maske_change) {
                  maske_change = 0;
                  printf_lcd(0,1,"<PIN2-Line>",1);
                  printf_lcd(0,2,"<PIN3-Line>",1);
                }
                if (temp_logset_index > index_alt) {
                  // Neues Zeichen
                  for (lcd_x = 0; lcd_x < SPALTEN_MAX; lcd_x++) {
                    // 2. Zeile scrollen (PIN2)
                    write_lcd(lcd_x + 0xA0 + 1,1); // Cursor Quellzeile
                    lcd_temp = read_lcd(0);        // Zeichen auslesen
                    write_lcd(lcd_x + 0xA0,1);     // Cursor Zielzeile
                    write_lcd(lcd_temp,0);         // Zeichen schreiben
                    // 3. Zeile scrollen (PIN3)
                    write_lcd(lcd_x + 0xC0 + 1,1); // Cursor Quellzeile
                    lcd_temp = read_lcd(0);        // Zeichen auslesen
                    write_lcd(lcd_x + 0xC0,1);     // Cursor Zielzeile
                    write_lcd(lcd_temp,0);         // Zeichen schreiben
                  }
                  if (temp_status & 0x02) {
                    write_byte_lcd(19,1,temp_rxd_pin2);
                  } else {
                    write_byte_lcd(19,1,' ');
                  }
                  if (temp_status & 0x01) {
                    write_byte_lcd(19,2,temp_rxd_pin3);
                  } else {
                    write_byte_lcd(19,2,' ');
                  }
                }
                index_alt = temp_logset_index;
                printf_lcd(0,3," \x18     F1   F2   F3",1);
                break;
        case 2: // Maske 3 (F2)
                if (maske_change) {
                  maske_change = 0;
                  printf_lcd(0,3,"<PIN2/3 Monitoring>",1);
                }
                if (temp_logset_index > index_alt) {
                  if (temp_status & 0x02) {
                    lcd_monitor_byte(temp_rxd_pin2);
                  }
                  if (temp_status & 0x01) {
                    lcd_monitor_byte(temp_rxd_pin3);
                  }
                  index_alt = temp_logset_index;
                }
                break;
      }
    }
    led_set(ledimage);          // LEDs aktualisieren
    taste = keypressed();       // Tastenabfrage
    if (taste == ESC) break;    // Abbruch durch Benutzer
    if (temp_logset_index) {
      // mind. 1 Byte wurde empfangen
      switch (taste) {
        case SETM :   // Anzeige-Maske 1
                      maske = 1;
                      maske_change = 1;
                      clear_lcd();
                      break;
        case SETP :   // Anzeige-Maske 1
                      maske = 2;
                      maske_change = 1;
                      clear_lcd();
                      break;
        case ET :     // Anzeige-Maske 3
                      maske = 0;
                      maske_change = 1;
                      clear_lcd();
                      break;
      }
    }
  }
  // Ende
  if (!logset.enable) {
    // Buffer voll
    clear_lcd();
    printf_lcd(0,0,"Buffer voll...",1);
    _delay_ms(2000);
    logset.index++;
    awd.uart_app_mode = 0;
  }
  logset.enable = 0;
  if (temp_logset_index) {
    // mind. 1 Byte im Speicher
    clear_lcd();
    printf_lcd(0,0,"<Logging beendet!>",1);
    sprintf(&lcd.buffer[0],"Es sind %d Byte",logset.index);
    printf_lcd(0,1,lcd.buffer,1);
    printf_lcd(0,2,"im Speicher.",1);
    printf_lcd(0,3," \x18   <DEL>        \x1C",1);

    taste = get_taste(TASTEN_NOREPEAT);
    if (taste == SETM) {
      // Lschen
      memset(&logbuch,0,sizeof(logbuch));
      logset.index      = 0;
      clear_lcd();
      printf_lcd(0,0,"<BUFFER GEL\x5CSCHT>",1);
      _delay_ms(1000);
    }
  }
}

// ********************************************************************
// Anzeigeprogramm fr die geloggten Daten im Speicher
// ********************************************************************
void display_daten(void) {
unsigned char taste             = 0;
unsigned char prg_aktiv         = 1;
unsigned char temp_rxd_pin2     = 0;
unsigned char temp_rxd_pin3     = 0;
unsigned char temp_rxd_pin2_c   = 0;
unsigned char temp_rxd_pin3_c   = 0;
unsigned char temp_status       = 0;
unsigned long temp_timecode     = 0;
signed int    temp_logset_index = 1; // (mit Vorzeichen)
unsigned int  index_offset      = 1; // Offset fr das Durchblttern 1/10/100/1000

  clear_lcd();
  taste_last = 5;   // Letzten Tastencode lschen
  if (!logset.index) {
    // Speicher leer
    printf_lcd(0,0,"! Noch kein Daten-",1);
    printf_lcd(0,1,"  satz im Speicher !",1);
    _delay_ms(1000);
    return;
  }
  // Mindestens ein Datensatz im Speicher
  while(prg_aktiv) {
    sprintf(&lcd.buffer[0],"Mode: %1d %d/%d",awd.uart_log_mode,temp_logset_index,logset.index);
    printf_lcd(0,0,lcd.buffer,1);
    switch (awd.uart_log_mode) {
      case 0: // Log-Mode 0
              temp_rxd_pin2   = logbuch.mode0[temp_logset_index - 1].rxd_pin2;
              temp_rxd_pin3   = logbuch.mode0[temp_logset_index - 1].rxd_pin3;
              temp_status     = logbuch.mode0[temp_logset_index - 1].status;
              temp_timecode   = logbuch.mode0[temp_logset_index - 1].timecode;
              break;
      case 1: // Log-Mode 1
              temp_rxd_pin2   = logbuch.mode1[temp_logset_index - 1].rxd_pin2;
              temp_rxd_pin3   = logbuch.mode1[temp_logset_index - 1].rxd_pin3;
              temp_status     = logbuch.mode1[temp_logset_index - 1].status;
              temp_timecode   = 0;
              break;
      case 2: // Log-Mode 2
              temp_rxd_pin2   = logbuch.mode2[temp_logset_index - 1].rxd_pin2;
              temp_rxd_pin3   = logbuch.mode2[temp_logset_index - 1].rxd_pin3;
              temp_status     = 0;
              temp_timecode   = logbuch.mode2[temp_logset_index - 1].timecode;
              break;
      case 3: // Log-Mode 3
              temp_rxd_pin2   = logbuch.mode3[temp_logset_index - 1].rxd_pin2;
              temp_rxd_pin3   = logbuch.mode3[temp_logset_index - 1].rxd_pin3;
              temp_status     = 0;
              temp_timecode   = 0;
              break;
      case 4: // Log-Mode 4
              temp_rxd_pin2   = logbuch.mode4[temp_logset_index - 1].rxd_pin2;
              temp_rxd_pin3   = 0;
              temp_status     = 0;
              temp_timecode   = 0;
              break;
      case 5: // Log-Mode 5
              temp_rxd_pin2   = 0;
              temp_rxd_pin3   = logbuch.mode5[temp_logset_index - 1].rxd_pin3;
              temp_status     = 0;
              temp_timecode   = 0;
              break;
    }
    // ASCII Einschrnkungen
    if ((temp_rxd_pin2 < 0x20) || (temp_rxd_pin2 > 0x7F)) temp_rxd_pin2_c = 0xDD; else temp_rxd_pin2_c = temp_rxd_pin2;
    if ((temp_rxd_pin3 < 0x20) || (temp_rxd_pin3 > 0x7F)) temp_rxd_pin3_c = 0xDD; else temp_rxd_pin3_c = temp_rxd_pin3;
    // Timecode und Status
    switch(awd.uart_log_mode) {
      case 0: // Log-Mode 0
              sprintf(&lcd.buffer[0],"T:%ldms S:%02X",temp_timecode,temp_status);
              break;
      case 1: // Log-Mode 1
              sprintf(&lcd.buffer[0],"T:-------- S:%02X",temp_status);
              break;
      case 2: // Log-Mode 2
              sprintf(&lcd.buffer[0],"T:%ldms S:--",temp_timecode);
              break;
      case 3: // Log-Mode 3
      case 4: // Log-Mode 4
      case 5: // Log-Mode 5
              sprintf(&lcd.buffer[0],"T:-------- S:--");
              break;
    }
    printf_lcd(0,1,lcd.buffer,1);
    // PIN2 / PIN3
    switch(awd.uart_log_mode) {
      case 0: // Log-Mode 0
      case 1: // Log-Mode 1
              if (temp_status & 0x02) {
                sprintf(&lcd.buffer[0],"PIN2:%02X\xDF%c ",temp_rxd_pin2,temp_rxd_pin2_c);
              } else {
                sprintf(&lcd.buffer[0],"PIN2:-- - ");
              }
              printf_lcd(0,2,lcd.buffer,0);
              if (temp_status & 0x01) {
                sprintf(&lcd.buffer[0],"PIN3:%02X\xDF%c ",temp_rxd_pin3,temp_rxd_pin3_c);
              } else {
                sprintf(&lcd.buffer[0],"PIN3:-- - ");
              }
              printf_lcd(10,2,lcd.buffer,0);
              break;
     case 2: // Log-Mode 2
     case 3: // Log-Mode 3
             sprintf(&lcd.buffer[0],"PIN2:%02X\xDF%c ",temp_rxd_pin2,temp_rxd_pin2_c);
             printf_lcd(0,2,lcd.buffer,0);
             sprintf(&lcd.buffer[0],"PIN3:%02X\xDF%c ",temp_rxd_pin3,temp_rxd_pin3_c);
             printf_lcd(10,2,lcd.buffer,0);
             break;
     case 4: // Log-Mode 4
             sprintf(&lcd.buffer[0],"PIN2:%02X\xDF%c ",temp_rxd_pin2,temp_rxd_pin2_c);
             printf_lcd(0,2,lcd.buffer,0);
             sprintf(&lcd.buffer[0],"PIN3:-- - ");
             printf_lcd(10,2,lcd.buffer,0);
             break;
     case 5: // Log-Mode 5
             sprintf(&lcd.buffer[0],"PIN2:-- - ");
             printf_lcd(0,2,lcd.buffer,0);
             sprintf(&lcd.buffer[0],"PIN3:%02X\xDF%c ",temp_rxd_pin3,temp_rxd_pin3_c);
             printf_lcd(10,2,lcd.buffer,0);
             break;
    }
    // Letzte Zeile
    sprintf(&lcd.buffer[0]," \x18     \x13    \x12   %04d",index_offset);
    printf_lcd(0,3,lcd.buffer,1);
    taste = get_taste(TASTEN_REPEAT);   // Tastenabfrage
    _delay_ms(50);
    switch (taste) {
      case ESC :          // Abbruch durch Benutzer
      case KEY_TIMEOUT :  // Timeout
                          prg_aktiv = 0;    // Programm-Ende
                          break;
      case ET :           // Offsetwert erhhen oder verringern (1/10/100/1000, dann wieder von 1 an...)
                          index_offset *= 10;
                          if (index_offset > 1000) index_offset = 1;
                          break;
      case SETP :         // vorwrts
                          temp_logset_index += index_offset;
                          break;
      case SETM :         // rckwrts
                          temp_logset_index -= index_offset;
                          break;
    }
    if (temp_logset_index < 1) temp_logset_index = 1;                       // Begrenzung nach unten
    if (temp_logset_index > logset.index) temp_logset_index = logset.index; // Begrenzung nach oben
  }
  _delay_ms(500);
}

// ********************************************************************
// USB-Stick Speicherprogramm fr die geloggten Daten im Speicher
// Rckgabe: 0x00 = Ok/Abbruch
//           0xXX = Fehler-Nr.
// ********************************************************************
unsigned char save_daten(void) {
unsigned char taste             = 0;    // Tastencode letzter Taste
unsigned char prg_aktiv         = 1;    // Datenerzeugung Schleife
unsigned char temp_rxd_pin2     = 0;    // Zwischenspeicher aktueller Datensatz
unsigned char temp_rxd_pin3     = 0;    // Zwischenspeicher aktueller Datensatz
unsigned char temp_rxd_pin2_c   = 0;    // Zwischenspeicher aktueller Datensatz (ASCII-Bereich)
unsigned char temp_rxd_pin3_c   = 0;    // Zwischenspeicher aktueller Datensatz (ASCII-Bereich)
unsigned char temp_status       = 0;    // Zwischenspeicher aktueller Datensatz
unsigned long temp_timecode     = 0;    // Zwischenspeicher aktueller Datensatz
unsigned int  temp_logset_index = 1;    // momentaner Datensatzindex
unsigned char zg_pos            = 0;    // aktuelle Schreibposition einer Datenzeile
unsigned char zg_anz            = 0;    // Lnge einer Datenzeile
unsigned char fb_raster         = 0;    // Fortschrittsbalken Aufteilung
unsigned char fb_x              = 0;    // Fortschrittsbalken Ausgabeposition
unsigned char fb_zg             = 0xD4; // Fortschrittsbalken Sonderzeichen-Kodierung

  clear_lcd();
  taste_last = 5;   // Letzten Tastencode lschen
  if (!logset.index) {
    // noch kein Datensatz im Speicher
    printf_lcd(0,0,"! Noch kein Daten-",1);
    printf_lcd(0,1,"  satz im Speicher !",1);
    _delay_ms(1000);
    return 0;
  }
  // Mindestens ein Datensatz befindet sich im Speicher und kann gespeichert werden.
  RE_VNC_L;         // Hardware-Reset VDIP1/VNC1L
  _delay_ms(100);   // Pause
  RE_VNC_H;         // VNC1L starten
  _delay_ms(100);   // Pause fr Anlauf

  // auf Vorhandensein eines VDIP1/VNC1L mit VDAP-Firmware prfen
  printf_lcd(0,0,"USB-Host VDIP1 Init",1);
  if (vnc_wait_for("VDAP")) return 20;
  _delay_ms(200); 
  printf_lcd(15,0,"Ok",1);

  // auf Vorhandensein eines USB-Sticks prfen bzw. auf Einstecken des USB-Sticks warten
  printf_lcd(0,1,"USB-Stick anstecken!",1);
  if (vnc_wait_for("Device Detected")) return 21;   // warten auf USB-Stick Erkennung
  vnc.status = vnc_prompt_check();                  // dann auf erstes Prompt (>) warten (Bereitschaft)
  if (vnc.status) return 22;                        // Fehlermeldung
  _delay_ms(100);
  printf_lcd(10,1,"Ok",1);

  // VDIP1/VNC1L Konfiguration "ECS"
  vnc.status = vnc_wr_cmd("ECS");                   // Extented Command Set
  if (vnc.status) return 23;                        // Fehlermeldung
  vnc.status = vnc_prompt_check();                  // Besttigung prfen  
  if (vnc.status) return 22;                        // Fehlermeldung
  _delay_ms(100);

  // VDIP1/VNC1L Konfiguration "IPA"
  vnc.status = vnc_wr_cmd("IPA");                   // Monitor Kommandos Zahlen in ASCII
  if (vnc.status) return 23;                        // Fehlermeldung
  vnc.status = vnc_prompt_check();                  // Besttigung prfen  
  if (vnc.status) return 22;                        // Fehlermeldung
  _delay_ms(1000);

  // Filenamen festlegen "ulog_ZZ.txt" bzw. "ulog_ZZ.bin" (ZZ Zhler von 00 ... 99)
  clear_lcd();
  printf_lcd(0,0,"USB-Stick",1);
  printf_lcd(0,3," \x18     \x13    \x12     \x1C",1);
  while (taste != ET) {
    sprintf(&lcd.buffer[0],"Filename:ulog\xC4%02d.%s",vnc.file_index,awd.stick_dateiformat ? "bin" : "txt");
    printf_lcd(0,1,lcd.buffer,1);
    blink_lcd(15,1,1);      // Blinken ein
    taste = get_taste(TASTEN_REPEAT);   // Tastenabfrage
    if (taste == SETP) { if (vnc.file_index < 98) vnc.file_index++; else vnc.file_index = 0; }
    if (taste == SETM) { if (vnc.file_index > 0 ) vnc.file_index--; else vnc.file_index = 99; }
    if (taste == ESC) {
      // Abbruch ohne Speicherung
      printf_lcd(0,3,"<Aktion abgebrochen>",1);
      blink_lcd(19,3,1);    // Blinken ein
      _delay_ms(2000);
      return 0;
    }
  }
  blink_lcd(14,1,0);        // Blinken aus

  // File auf dem USB-Stick anlegen
  // Falls File mit gleichem Namen bereits vorhanden ist, wird dieses ohne Nachfrage berschrieben!
  sprintf(&vnc.send_buffer[0],"OPW ulog_%02d.%s",vnc.file_index,awd.stick_dateiformat ? "bin" : "txt");
  if (vnc_wr_cmd(vnc.send_buffer)) return 24;       // Fehler Kmdo. wurde nicht angenommen
  vnc.status = vnc_rd_answer(PROMPT_UND_ERROR_CHECK);
  if (vnc.status != PROMPT_OK) return 22;           // Fehlermeldung

  // Schreibposition im File auf Anfang setzen
  if (vnc_wr_cmd("SEK 0")) return 24;               // Fehler Kmdo. wurde nicht angenommen
  vnc.status = vnc_rd_answer(PROMPT_UND_ERROR_CHECK);
  if (vnc.status != PROMPT_OK) return 22;           // Fehlermeldung

  // Info/Kopfzeile erzeugen
  if (!awd.stick_dateiformat) {
    // txt/csv Dateiformat
    switch (awd.uart_log_mode) {
      case 0: // Log-Mode 0 Infozeile
              sprintf(&vnc.send_buffer[0],"ULOG Mode: 0\x0D\x0ANr.;Timcecode;Pin2H;Pin2C;Pin3H;Pin3C;Status\x0D\x0A");
              break;
      case 1: // Log-Mode 1 Infozeile
              sprintf(&vnc.send_buffer[0],"ULOG Mode: 1\x0D\x0ANr.;Pin2H;Pin2C;Pin3H;Pin3C;Status\x0D\x0A");
              break;
      case 2: // Log-Mode 2 Infozeile
              sprintf(&vnc.send_buffer[0],"ULOG Mode: 2\x0D\x0ANr.;Timcecode;Pin2H;Pin2C;Pin3H;Pin3C\x0D\x0A");
              break;
      case 3: // Log-Mode 3 Infozeile
              sprintf(&vnc.send_buffer[0],"ULOG Mode: 3\x0D\x0ANr.;Pin2H;Pin2C;Pin3H;Pin3C\x0D\x0A");
              break;
      case 4: // Log-Mode 4 Infozeile
              sprintf(&vnc.send_buffer[0],"ULOG Mode: 4\x0D\x0ANr.;Pin2H;Pin2C\x0D\x0A");
              break;
      case 5: // Log-Mode 5 Infozeile
              sprintf(&vnc.send_buffer[0],"ULOG Mode: 5\x0D\x0ANr.;Pin3H;Pin3C\x0D\x0A");
              break;
    }

    // Info/Kopfzeile schreiben vorbereiten (Lngeninformation)
    sprintf (&vnc.temp_buffer[0],"WRF %u",strlen(vnc.send_buffer)); // Zeilenlnge
    if (vnc_wr_cmd(vnc.temp_buffer)) return 24;                     // Fehler Kmdo. wurde nicht angenommen
    // hier keine Prompt-Prfung!!!
  } else {
    // bin-Dateiformat
    sprintf(&vnc.send_buffer[0],"ULOG V:%02u M:%02u\x0D\x0A",awd.software_version,awd.uart_log_mode); // Infozeile

    // Info/Kopfzeile schreiben vorbereiten (Lngeninformation)
    if (vnc_wr_cmd("WRF 16")) return 24;                    // Fehler Kmdo. wurde nicht angenommen
    // hier keine Prompt-Prfung!!!
  }
  
  // Info/Kopfzeile Inhalt schreiben
  for(zg_pos = 0; zg_pos < strlen(vnc.send_buffer); zg_pos++) {
      if (vnc_wr_byte(vnc.send_buffer[zg_pos])) return 24;  // Fehler Byte wurde nicht angenommen
      if (awd.stick_speed) _delay_ms(1);                    // USB-Speed/Timing bremsen bei Bedarf
  }
 
  // Promptprfung, ob Kommando komplett angenommen wurde
  vnc.status = vnc_rd_answer(PROMPT_UND_ERROR_CHECK);
  if (vnc.status != PROMPT_OK) return 22;                   // Fehlermeldung

  // Ausgabe der geloggten Datenstze, je nach Aufzeichnungs-Mode
  printf_lcd(0,2,"Speichervorgang...",1);
  printf_lcd(0,3,"                100%",1);
  fb_raster = logset.index / 80 + 1;    // Rasterung fr den Fortschrittsbalken (80 Stufen) berechnen

  // Datenzeilen erzeugen, Abschnitt wird fr jeden Datensatz 1 mal durchlaufen
  while(prg_aktiv) {
    // Fortschrittsbalkenanzeige aktualisieren
    if (!(temp_logset_index % fb_raster)) {
      // Balken fortschreiben (Balkenelemente liegen im ROM des Displays, Code: 0xD4|,D3||,D2|||,D1||||,D0|||||)
      write_byte_lcd(fb_x,3,fb_zg);     // Balkenelement ausgeben
      fb_zg--;                          // nchstes Balkenelement vorbereiten
      if (fb_zg < 0xD0) {
        fb_zg = 0xD4;                   // Balkenelement rcksetzen
        fb_x++;                         // Display-Position erhhen
      }
    }
    // Datensatz temporr zwischenspeichern
    switch (awd.uart_log_mode) {
      case 0: // Log-Mode 0
              temp_rxd_pin2   = logbuch.mode0[temp_logset_index - 1].rxd_pin2;
              temp_rxd_pin3   = logbuch.mode0[temp_logset_index - 1].rxd_pin3;
              temp_status     = logbuch.mode0[temp_logset_index - 1].status;
              temp_timecode   = logbuch.mode0[temp_logset_index - 1].timecode;
              break;
      case 1: // Log-Mode 1
              temp_rxd_pin2   = logbuch.mode1[temp_logset_index - 1].rxd_pin2;
              temp_rxd_pin3   = logbuch.mode1[temp_logset_index - 1].rxd_pin3;
              temp_status     = logbuch.mode1[temp_logset_index - 1].status;
              temp_timecode   = 0;
              break;
      case 2: // Log-Mode 2
              temp_rxd_pin2   = logbuch.mode2[temp_logset_index - 1].rxd_pin2;
              temp_rxd_pin3   = logbuch.mode2[temp_logset_index - 1].rxd_pin3;
              temp_status     = 0;
              temp_timecode   = logbuch.mode2[temp_logset_index - 1].timecode;
              break;
      case 3: // Log-Mode 3
              temp_rxd_pin2   = logbuch.mode3[temp_logset_index - 1].rxd_pin2;
              temp_rxd_pin3   = logbuch.mode3[temp_logset_index - 1].rxd_pin3;
              temp_status     = 0;
              temp_timecode   = 0;
              break;
      case 4: // Log-Mode 4
              temp_rxd_pin2   = logbuch.mode4[temp_logset_index - 1].rxd_pin2;
              temp_rxd_pin3   = 0;
              temp_status     = 0;
              temp_timecode   = 0;
              break;
      case 5: // Log-Mode 5
              temp_rxd_pin2   = 0;
              temp_rxd_pin3   = logbuch.mode5[temp_logset_index - 1].rxd_pin3;
              temp_status     = 0;
              temp_timecode   = 0;
              break;
    }
    // ASCII Einschrnkungen (0x3B ; als CSV-Trennzeichen)
    if ((temp_rxd_pin2 < 0x20) || (temp_rxd_pin2 > 0x7F) || (temp_rxd_pin2 == 0x3B)) temp_rxd_pin2_c = '.'; else temp_rxd_pin2_c = temp_rxd_pin2;
    if ((temp_rxd_pin3 < 0x20) || (temp_rxd_pin3 > 0x7F) || (temp_rxd_pin3 == 0x3B)) temp_rxd_pin3_c = '.'; else temp_rxd_pin3_c = temp_rxd_pin3;

    // Datenzeile erzeugen
    if (!awd.stick_dateiformat) {
      // txt/csv Dateiformat
      switch(awd.uart_log_mode) {
        case 0: // Log-Mode 0
                sprintf(&vnc.send_buffer[0],"%04u;",temp_logset_index);
                sprintf(&vnc.send_buffer[5],"%010lu;",temp_timecode);
                if (temp_status & 0x02) {
                    sprintf(&vnc.send_buffer[16],"%02X;%c;",temp_rxd_pin2,temp_rxd_pin2_c);
                } else {
                    sprintf(&vnc.send_buffer[16],"--;-;");
                }
                if (temp_status & 0x01) {
                    sprintf(&vnc.send_buffer[21],"%02X;%c;",temp_rxd_pin3,temp_rxd_pin3_c);
                } else {
                    sprintf(&vnc.send_buffer[21],"--;-;");
                }
                sprintf(&vnc.send_buffer[26],"%02X",temp_status);
                vnc.send_buffer[28] = 0x0D;   // CR fr Zeilenumbruch
                vnc.send_buffer[29] = 0x0A;
                vnc.send_buffer[30] = 0x00;   // Null-Terminierung als String-Ende-Kennung
                break;
        case 1: // Log-Mode 1
                sprintf(&vnc.send_buffer[0],"%04u;",temp_logset_index);
                if (temp_status & 0x02) {
                    sprintf(&vnc.send_buffer[5],"%02X;%c;",temp_rxd_pin2,temp_rxd_pin2_c);
                } else {
                    sprintf(&vnc.send_buffer[5],"--;-;");
                }
                if (temp_status & 0x01) {
                    sprintf(&vnc.send_buffer[10],"%02X;%c;",temp_rxd_pin3,temp_rxd_pin3_c);
                } else {
                    sprintf(&vnc.send_buffer[10],"--;-;");
                }
                sprintf(&vnc.send_buffer[15],"%02X",temp_status);
                vnc.send_buffer[17] = 0x0D;   // CR fr Zeilenumbruch
                vnc.send_buffer[18] = 0x0A;
                vnc.send_buffer[19] = 0x00;   // Null-Terminierung als String-Ende-Kennung
                break;
        case 2: // Log-Mode 2
                sprintf(&vnc.send_buffer[0],"%04u;",temp_logset_index);
                sprintf(&vnc.send_buffer[5],"%010lu;",temp_timecode);
                sprintf(&vnc.send_buffer[16],"%02X;%c;",temp_rxd_pin2,temp_rxd_pin2_c);
                sprintf(&vnc.send_buffer[21],"%02X;%c;",temp_rxd_pin3,temp_rxd_pin3_c);
                vnc.send_buffer[26] = 0x0D;   // CR fr Zeilenumbruch
                vnc.send_buffer[27] = 0x0A;
                vnc.send_buffer[28] = 0x00;   // Null-Terminierung als String-Ende-Kennung
                break;
        case 3: // Log-Mode 3
                sprintf(&vnc.send_buffer[0],"%04u;",temp_logset_index);
                sprintf(&vnc.send_buffer[5],"%02X;%c;",temp_rxd_pin2,temp_rxd_pin2_c);
                sprintf(&vnc.send_buffer[10],"%02X;%c;",temp_rxd_pin3,temp_rxd_pin3_c);
                vnc.send_buffer[15] = 0x0D;   // CR fr Zeilenumbruch
                vnc.send_buffer[16] = 0x0A;
                vnc.send_buffer[17] = 0x00;   // Null-Terminierung als String-Ende-Kennung
                break;
        case 4: // Log-Mode 4
                sprintf(&vnc.send_buffer[0],"%04u;",temp_logset_index);
                sprintf(&vnc.send_buffer[5],"%02X;%c;",temp_rxd_pin2,temp_rxd_pin2_c);
                vnc.send_buffer[10] = 0x0D;   // CR fr Zeilenumbruch
                vnc.send_buffer[11] = 0x0A;
                vnc.send_buffer[12] = 0x00;   // Null-Terminierung als String-Ende-Kennung
                break;
        case 5: // Log-Mode 5
                sprintf(&vnc.send_buffer[0],"%04u;",temp_logset_index);
                sprintf(&vnc.send_buffer[5],"%02X;%c;",temp_rxd_pin3,temp_rxd_pin3_c);
                vnc.send_buffer[10] = 0x0D;   // CR fr Zeilenumbruch
                vnc.send_buffer[11] = 0x0A;
                vnc.send_buffer[12] = 0x00;   // Null-Terminierung als String-Ende-Kennung
                break;
      }

      // Zeile auf den USB-Stick schreiben
      sprintf (&vnc.temp_buffer[0],"WRF %u",strlen(vnc.send_buffer)); // Zeilenlnge
      if (vnc_wr_cmd(vnc.temp_buffer)) return 24;                     // Fehler Kmdo. wurde nicht angenommen
      // hier keine Prompt-Prfung!!!
      for(zg_pos = 0; zg_pos < strlen(vnc.send_buffer); zg_pos++) {
        if (vnc_wr_byte(vnc.send_buffer[zg_pos])) return 24;          // Fehler Byte wurde nicht angenommen
        if (awd.stick_speed) _delay_ms(1);                            // USB-Speed/Timing bremsen bei Bedarf
      }

      // Promptprfung, ob Zeile komplett angenommen wurde
      vnc.status = vnc_rd_answer(PROMPT_UND_ERROR_CHECK);
      if (vnc.status != PROMPT_OK) return 22;                         // Fehlermeldung
      if (awd.stick_speed) _delay_ms(5);                              // USB-Speed/Timing bremsen bei Bedarf

    } else {
      // bin Dateiformat
      switch(awd.uart_log_mode) {
        case 0: // Log-Mode 0
                // Datenzeile generieren (TIMECODE 4-Byte, PIN2 1-Byte, PIN 3 1-Byte, STATUS 1-Byte)
                vnc.send_buffer[0] = (unsigned char) temp_timecode         & 0x000000FF;    // LSB
                vnc.send_buffer[1] = (unsigned char) (temp_timecode >> 8)  & 0x000000FF;
                vnc.send_buffer[2] = (unsigned char) (temp_timecode >> 16) & 0x000000FF;
                vnc.send_buffer[3] = (unsigned char) (temp_timecode >> 24) & 0x000000FF;    // MSB
                vnc.send_buffer[4] = temp_rxd_pin2;
                vnc.send_buffer[5] = temp_rxd_pin3;
                vnc.send_buffer[6] = temp_status;
                zg_anz = 7; // Eintragslnge
                break;
        case 1: // Log-Mode 1
                // Datenzeile generieren (PIN2 1-Byte, PIN 3 1-Byte, STATUS 1-Byte)
                vnc.send_buffer[0] = temp_rxd_pin2;
                vnc.send_buffer[1] = temp_rxd_pin3;
                vnc.send_buffer[2] = temp_status;
                zg_anz = 3; // Eintragslnge
                break;
        case 2: // Log-Mode 2
                // Datenzeile generieren (TIMECODE 4-Byte, PIN2 1-Byte, PIN 3 1-Byte)
                vnc.send_buffer[0] = (unsigned char) temp_timecode         & 0x000000FF;    // LSB
                vnc.send_buffer[1] = (unsigned char) (temp_timecode >> 8)  & 0x000000FF;
                vnc.send_buffer[2] = (unsigned char) (temp_timecode >> 16) & 0x000000FF;
                vnc.send_buffer[3] = (unsigned char) (temp_timecode >> 24) & 0x000000FF;    // MSB
                vnc.send_buffer[4] = temp_rxd_pin2;
                vnc.send_buffer[5] = temp_rxd_pin3;
                zg_anz = 6; // Eintragslnge
                break;
        case 3: // Log-Mode 3
                // Datenzeile generieren (PIN2 1-Byte, PIN 3 1-Byte)
                vnc.send_buffer[0] = temp_rxd_pin2;
                vnc.send_buffer[1] = temp_rxd_pin3;
                zg_anz = 2; // Eintragslnge
                break;
        case 4: // Log-Mode 4
                // Datenzeile generieren (PIN2 1-Byte)
                vnc.send_buffer[0] = temp_rxd_pin2;
                zg_anz = 1; // Eintragslnge
                break;
        case 5: // Log-Mode 5
                // Datenzeile generieren (PIN3 1-Byte)
                vnc.send_buffer[0] = temp_rxd_pin3;
                zg_anz = 1; // Eintragslnge
                break;
      }

      // Zeile auf dem USB-Stick schreiben
      sprintf (&vnc.temp_buffer[0],"WRF %u",zg_anz);                // Zeilenlnge
      if (vnc_wr_cmd(vnc.temp_buffer)) return 24;                   // Fehler Kmdo. wurde nicht angenommen
      // hier keine Prompt-Prfung!!!
      for(zg_pos = 0; zg_pos < zg_anz; zg_pos++) {
        if (vnc_wr_byte(vnc.send_buffer[zg_pos])) return 24;        // Fehler Byte wurde nicht angenommen
        if (awd.stick_speed) _delay_ms(1);                          // USB-Speed/Timing bremsen bei Bedarf
      }

      // Promptprfung, ob Zeile komplett angenommen wurde
      vnc.status = vnc_rd_answer(PROMPT_UND_ERROR_CHECK);
      if (vnc.status != PROMPT_OK) return 22;     // Fehlermeldung
      if (awd.stick_speed) _delay_ms(5);          // USB-Speed/Timing bremsen bei Bedarf

    }

    // nchster Datensatz
    temp_logset_index++;
    if (temp_logset_index > logset.index) prg_aktiv = 0;    // Endeprfung
    if (awd.stick_speed) _delay_ms(5);                      // USB-Speed/Timing bremsen bei Bedarf
  }

  // File schlieen
  sprintf (&vnc.send_buffer[0],"CLF ulog_%02d.%s",vnc.file_index,awd.stick_dateiformat ? "bin" : "txt");
  if (vnc_wr_cmd(vnc.send_buffer)) return 24;   // Fehler Kmdo. wurde nicht angenommen
  if (awd.stick_speed) _delay_ms(100);          // USB-Speed/Timing bremsen bei Bedarf 

  // Promptprfung, ob Kommando komplett angenommen wurde
  vnc.status = vnc_rd_answer(PROMPT_UND_ERROR_CHECK);
  if (vnc.status != PROMPT_OK) return 22;
  
  // Erfolgsanzeige
  printf_lcd(0,2," ",1);
  printf_lcd(0,3,"-> Speichern Ok",1);
  _delay_ms(2000);

  return 0; // Ende
}


// ********************************************************************
// Auswahlprogramm fr den Log-Mode
// (PIN2=RxD=rxd_pin2=UART1, PIN3=TxD=rxd_pin3=UART0)
// Mode 0: PIN2, PIN3, Status, Timecode
// Mode 1: PIN2, PIN3, Status
// Mode 2: PIN2, PIN3, Timecode
// Mode 3: PIN2, PIN3
// Mode 4: PIN2
// Mode 5: PIN3
// ********************************************************************
void log_mode_auswahl(void) {
unsigned char taste;
unsigned char menu_POS           = 0;
unsigned char temp_uart_log_mode = 0;

// Anzeige-Texte
char *menu_TXT[] = {
   /*  0 */ "\x10 0 PIN2/3-Stat-Time",
   /*  1 */ "\x10 1 PIN2/3-Status",
   /*  2 */ "\x10 2 PIN2/3-Time",
   /*  3 */ "\x10 3 PIN2/3",
   /*  4 */ "\x10 4 nur PIN2",
   /*  5 */ "\x10 5 nur PIN3" 
};

  temp_uart_log_mode = awd.uart_log_mode;  // Momentan eingestellter Log-Mode
  menu_POS           = temp_uart_log_mode; // Menpos einstellen
  while(1) {
    clear_lcd();
    printf_lcd(0,0,"UART Datenlogger",1);
    printf_lcd(0,1,"Log-Mode Auswahl:",1);
    printf_lcd(0,3," \x18     \x13    \x12     \x1C",1);
    while(1) {
      printf_lcd(0,2,menu_TXT[menu_POS],1);
      blink_lcd(0,2,1);
      taste = get_taste(TASTEN_NOREPEAT);
      _delay_ms(50);
      if ((taste == ESC) || (taste == ET) || (taste == KEY_TIMEOUT)) break; // ESC oder ET oder Timeout
      if (taste == SETP) { if (menu_POS < 5) menu_POS++; else menu_POS = 0; }
      if (taste == SETM) { if (menu_POS > 0) menu_POS--; else menu_POS = 5; }
    };
    blink_lcd(0,2,0);
    if ((taste == ESC) || (taste == KEY_TIMEOUT)) break; // ESC oder Timeout
    // Taste ET
    switch (menu_POS) {
      case 0:   // Mode 0
                awd.uart_log_mode = 0;
                logset.max        = LOGBUCH0_MAX;
                break;
      case 1:   // Mode 1
                awd.uart_log_mode = 1;
                logset.max        = LOGBUCH1_MAX;
                break;
      case 2:   // Mode 2
                awd.uart_log_mode = 2;
                logset.max        = LOGBUCH2_MAX;
             break;
      case 3:   // Mode 3
                awd.uart_log_mode = 3;
                logset.max        = LOGBUCH3_MAX;
             break;
      case 4:   // Mode 4
                awd.uart_log_mode = 4;
                logset.max        = LOGBUCH4_MAX;
             break;
      case 5:   // Mode 5
                awd.uart_log_mode = 5;
                logset.max        = LOGBUCH5_MAX;
                break;
    }
    if (awd.uart_log_mode != temp_uart_log_mode) {
      // Neuer Log-Mode eingestellt
      // Speicher lschen und rcksetzen
      memset(&logbuch,0,sizeof(logbuch));
      logset.index = 0;
      // Parameterdaten in EEPROM speichern
      eeprom_write_block((void*)&awd,&awd_ee,sizeof(awd));
    }
    sprintf(&lcd.buffer[0],"MODE: %1d eingestellt!",awd.uart_log_mode);
    printf_lcd(0,3,lcd.buffer,1);
    _delay_ms(1000);
    break;
  }
}

// ***********************************************************************
// Programm Einstellungen
// ***********************************************************************
void einstellungen(void) {
unsigned char param_POS          = 0;
unsigned char wert_POS           = 0;
unsigned char taste              = 0;
unsigned char wert_b             = 0;
unsigned char wert_max           = 0;
unsigned char wert_min           = 0;
unsigned char wert_delta         = 0;
unsigned char aenderung_symbol   = ' ';
unsigned char aenderung          = 0;
unsigned char uart_log_mode_temp = 0;

// Anzeige-Texte
char *txt_fkt_1[] = { "150","300","600","1200","2400","4800","9600","14400","19200","38400","57600","76800","115200","230400" };
char *txt_fkt_2[] = { "5","6","7","8","9"};
char *txt_fkt_3[] = { "Keine","Even","Odd" };
char *txt_fkt_4[] = { "Normal","Invertiert" };
char *txt_fkt_5[] = { "Mode 0","Mode 1","Mode 2","Mode 3","Mode 4","Mode 5" };
char *txt_fkt_6[] = { "New","Append" };
char *txt_fkt_7[] = { "Text/CSV","Bin\x7Br" };
char *txt_fkt_8[] = { "Schnell","Langsam" };
char *txt_fkt_9[] = { "Aus","Ein","Automatik" };

  uart_log_mode_temp = awd.uart_log_mode;
  clear_lcd();
  printf_lcd(0,0,"Einstellungen:",1);
  printf_lcd(0,3," \x18     \x13    \x12     \x1C",1);
  while(1) {
    sprintf(&lcd.buffer[0],"<%s>",parameter[param_POS].txt);
    printf_lcd(0,1,lcd.buffer,1);
    if (parameter[param_POS].aenderung) aenderung_symbol = ' '; else aenderung_symbol = '#';
    wert_min   = parameter[param_POS].min;
    wert_max   = parameter[param_POS].max;
    wert_delta = parameter[param_POS].delta;
    wert_b     = *parameter[param_POS].adresse;
    sprintf(&lcd.buffer[0],"P%03u =%c",parameter[param_POS].nr,aenderung_symbol);
    printf_lcd(0,2,lcd.buffer,1);
    switch (parameter[param_POS].funktion) {
      case 1:  sprintf(&lcd.buffer[0],"%s Baud",txt_fkt_1[wert_b]); break; // Baudrate
      case 2:  sprintf(&lcd.buffer[0],"%s Bit",txt_fkt_2[wert_b]);  break; // Datenbits
      case 3:  sprintf(&lcd.buffer[0],"%s",txt_fkt_3[wert_b]);      break; // Paritt
      case 4:  sprintf(&lcd.buffer[0],"%s",txt_fkt_4[wert_b]);      break; // Polaritt
      case 5:  sprintf(&lcd.buffer[0],"%s",txt_fkt_5[wert_b]);      break; // Log-Mode
      case 6:  sprintf(&lcd.buffer[0],"%s",txt_fkt_6[wert_b]);      break; // Aufzeichnungs-Mode
      case 7:  sprintf(&lcd.buffer[0],"%s",txt_fkt_7[wert_b]);      break; // USB-Stick Format
      case 8:  sprintf(&lcd.buffer[0],"%s",txt_fkt_8[wert_b]);      break; // USB-Stick Geschwindigkeit
      case 9:  sprintf(&lcd.buffer[0],"%s",txt_fkt_9[wert_b]);      break; // LCD Hintergrundbeleuchtung
      case 10: sprintf(&lcd.buffer[0],"%02u",wert_b);               break; // Firmwareversion
      default: sprintf(&lcd.buffer[0],"-");                         break; // Standard
    }
    printf_lcd(7,2,lcd.buffer,1);
    blink_lcd(0,2,1);
    taste = get_taste(TASTEN_NOREPEAT);
    if ((taste == ESC) || (taste == KEY_TIMEOUT)) break;
    if (taste == SETP) { if (param_POS < PARA_MAX) param_POS++; else param_POS = 0; }
    if (taste == SETM) { if (param_POS > 0) param_POS--; else param_POS = PARA_MAX; }
    if ((taste == ET) && parameter[param_POS].aenderung) {
      // Parameterwert ndern
      wert_POS = wert_b;
      while(1) {
        switch (parameter[param_POS].funktion) {
          case 1:  sprintf(&lcd.buffer[0],"%s Baud",txt_fkt_1[wert_POS]); break; // Baudrate
          case 2:  sprintf(&lcd.buffer[0],"%s Bit",txt_fkt_2[wert_POS]);  break; // Datenbits
          case 3:  sprintf(&lcd.buffer[0],"%s",txt_fkt_3[wert_POS]);      break; // Paritt
          case 4:  sprintf(&lcd.buffer[0],"%s",txt_fkt_4[wert_POS]);      break; // Polaritt
          case 5:  sprintf(&lcd.buffer[0],"%s",txt_fkt_5[wert_POS]);      break; // Log-Mode
          case 6:  sprintf(&lcd.buffer[0],"%s",txt_fkt_6[wert_POS]);      break; // Aufzeichnungs-Mode
          case 7:  sprintf(&lcd.buffer[0],"%s",txt_fkt_7[wert_POS]);      break; // USB-Stick Format
          case 8:  sprintf(&lcd.buffer[0],"%s",txt_fkt_8[wert_POS]);      break; // USB-Stick Geschwindigkeit
          case 9:  sprintf(&lcd.buffer[0],"%s",txt_fkt_9[wert_POS]);      break; // LCD Hintergrundbeleuchtung
          case 10: sprintf(&lcd.buffer[0],"%02u",wert_POS);               break; // Firmwareversion
          default: sprintf(&lcd.buffer[0],"-");                           break; // Standard
        }
        printf_lcd(7,2,lcd.buffer,1);
        blink_lcd(7,2,1);
        taste = get_taste(TASTEN_NOREPEAT);
        if (taste == SETP) { if (wert_POS < wert_max) wert_POS += wert_delta; else wert_POS = wert_min; }
        if (taste == SETM) { if (wert_POS > wert_min) wert_POS -= wert_delta; else wert_POS = wert_max; }
        if ((taste == ESC) || (taste == KEY_TIMEOUT)) break;
        if (taste == ET) {
          // nderung bernehmen
          aenderung = 1;
          *parameter[param_POS].adresse = wert_POS;
          break;
        }
      }
    }
  }
  if (aenderung) {
    // LCD Hintergrundbeleuchtung
    if (awd.lcd_hgbl) LCD_HGBL_EIN; else LCD_HGBL_AUS;    // LCD Beleuchtung
    led_set(ledimage);                                    // LEDs aktualisieren
    // UART
    if (awd.uart01_pol) POL_RS232_H; else POL_RS232_L;    // PIN2/3 Polaritt
    usart_init(F_CPU);                                    // UART initialisieren
    // Log-Mode
    if (uart_log_mode_temp != awd.uart_log_mode) {
      switch (awd.uart_log_mode) {
        case 0: logset.max = LOGBUCH0_MAX; break;   // Mode 0
        case 1: logset.max = LOGBUCH1_MAX; break;   // Mode 1
        case 2: logset.max = LOGBUCH2_MAX; break;   // Mode 2
        case 3: logset.max = LOGBUCH3_MAX; break;   // Mode 3
        case 4: logset.max = LOGBUCH4_MAX; break;   // Mode 4
        case 5: logset.max = LOGBUCH5_MAX; break;   // Mode 5
      }
      // Speicher lschen und Rcksetzen
      memset(&logbuch,0,sizeof(logbuch));
      logset.index = 0;
    }
    // Parameterdaten in EEPROM speichern
    cli();                  // Interrupts sperren
    eeprom_write_block((void*)&awd,&awd_ee,sizeof(awd));
    sei();                  // Interrupts freigeben
    printf_lcd(0,3,"-> gespeichert",1);
    blink_lcd(0,3,1);
    _delay_ms(2000);        // Pause, nur fr Infozwecken
  }
} 

// ********************************************************************
// Informationsanzeige
// ********************************************************************
void information (unsigned char abschluss) {
  clear_lcd();
  printf_lcd(0,0,"UART Datenlogger",1);
  printf_lcd(0,1,"AVR Atmega-1284P",1);
  printf_lcd(0,2,"M.Kahnt (c)2010",1);
  printf_lcd(0,3,"V1.0 / 20.10.2010",1);
  if (abschluss) get_taste(TASTEN_NOREPEAT);  // ggf. auf Tastendruck warten

}

// ********************************************************************
// Bedienprogramm (Hauptmen)
// ********************************************************************
void bedienung(void) {
unsigned char taste;
unsigned char menu_POS = 0;
unsigned char status   = 0;

// Anzeige-Texte
char *menu_TXT[] = {
  /* 0 */ "\x10 Logging starten",
  /* 1 */ "\x10 Mode ausw\x7Bhlen",
  /* 2 */ "\x10 Daten anzeigen",
  /* 3 */ "\x10 USB-Stick sichern",
  /* 4 */ "\x10 Einstellungen",
  /* 5 */ "\x10 Information" 
};

  while(1) {
    LED_RI_AUS;
    LED_DTR_AUS;
    LED_DSR_AUS;
    LED_RTS_AUS;
    LED_CTS_AUS;
    LED_D1_AUS;
    LED_D2_EIN;
    led_set(ledimage);
    clear_lcd();
    printf_lcd(0,0,"UART Datenlogger",1);
    printf_lcd(0,1,"System-Men\x5E:",1);
    printf_lcd(0,3," \x18     \x13    \x12     \x1C",1);
    while(1) {
      printf_lcd(0,2,menu_TXT[menu_POS],1);
      blink_lcd(0,2,1);
      taste = get_taste(TASTEN_NOREPEAT);
      _delay_ms(50);
      if ((taste == ESC) || (taste == ET) || (taste == KEY_TIMEOUT)) break; // ESC oder ET oder Timeout
      if (taste == SETP) { if (menu_POS < 5) menu_POS++; else menu_POS = 0; }
      if (taste == SETM) { if (menu_POS > 0) menu_POS--; else menu_POS = 5; }
    };
    blink_lcd(0,2,0);
    if ((taste == ESC) || (taste == KEY_TIMEOUT)) break; // ESC oder Timeout
    switch (menu_POS) {
      case 0: // Log starten
              logging();
              break;
      case 1: // Log-Mode auswhlen
              log_mode_auswahl();
              menu_POS = 0;
              break;
      case 2: // Log-Daten ansehen
              display_daten();
              menu_POS = 0;
           break;
      case 3: // Log-Daten auf USB-Stick speichern
              status = save_daten();
              if (status) vnc_fehlermeldung(status,5);
              menu_POS = 0;
           break;
      case 4: // Einstellungen
              einstellungen();
              menu_POS = 0;
              break;
      case 5: // Information
              information(1);
              menu_POS = 0;
              break;
    }
  }
  _delay_ms(500);
  clear_lcd();
  LED_D2_AUS;
  led_set(ledimage);
}

// ********************************************************************
// System-Initialisierungen
// ********************************************************************
void init_system (void) {
  // Portdefinition
  DDRB  &= ~((1<<PX0) | (1<<PX1) | (1<<PX2) | (1<<PX3)); // ET,SET+,SET-,ET (Eingnge)
  PORTB |=  ((1<<PX0) | (1<<PX1) | (1<<PX2) | (1<<PX3)); // Pull-Up-R zuschalten
  DDRB  |=   (1<<PX4);  // ENL (Ausgang)

  DDRC  &= ~((1<<PX0) | (1<<PX4) | (1<<PX5)); // JUM,RXF#,TXE# (Eingnge)
  PORTC |=  ((1<<PX0) | (1<<PX4) | (1<<PX5)); // Pull-Up-R zuschalten
  DDRC  |=  ((1<<PX1) | (1<<PX2) | (1<<PX3) | (1<<PX6) | (1<<PX7)) ;    // CSD,RSU,RSD,RD#,WR (Ausgnge)

  DDRD  &= ~((1<<PX3) | (1<<PX4) | (1<<PX5) | (1<<PX6) | (1<<PX7));     // RIN#,RTS#,CTS#,DTR#,DSR# (Eingnge)
  PORTD |=  ((1<<PX3) | (1<<PX4) | (1<<PX5) | (1<<PX6) | (1<<PX7));     // Pull-Up-R zuschalten
  DDRD  |=   (1<<PX1);  // POL (Ausgang)

  DDRA  &= ~(1<<PX2);   // VDIP1/VNC1L SPI: SDO (Eingang)
  PORTA |=  (1<<PX2);   // Pull-Up-R zuschalten
  DDRA  |= ((1<<PX0) | (1<<PX1) | (1<<PX3)) ; // VDIP1/VNC1L SPI: SCLK,SDI,CS (Ausgnge)

  CS_LCD_H;             // LCD deaktivieren
  EN_595_L;             // 74HC595 deaktivieren
  RES_LCD_H;            // LCD Resetpin setzen

  spi_master_init();    // SPI-Interface initialisieren
  init_lcd();           // LCD initialisieren
  vnc_init();           // VDIP1/VNC1L rcksetzen

  LED_LCD_AUS;          // alle LEDs aus
  led_set(ledimage);

  // Parameterdaten aus EEPROM laden
  eeprom_read_block((void*)&awd,&awd_ee,sizeof(awd));

  // Prfen auf Werkseinstellungen herstellen oder Erstinbetriebnahme
  if ((!JUMPER_JP1) || (awd.eeprom_code != 0x55)) {

     // Grundinitialisierungen erforderlich
    LCD_HGBL_EIN;                   // LCD Beleuchtung Ein
    led_set(ledimage);              // Schieberegister aktualisieren
    printf_lcd(0,0,"UART Datenlogger",1);
    printf_lcd(0,1,"-> System wird",1);
    printf_lcd(0,2,"   initialisiert...",1);
 
    awd.uart0_baudrate    = 6;      // PIN3 Einstellungen
    awd.uart0_parity      = 0;
    awd.uart0_datenbits   = 3;
    awd.uart1_baudrate    = 6;      // PIN2 Einstellungen
    awd.uart1_parity      = 0;
    awd.uart1_datenbits   = 3;
    awd.uart01_pol        = 1;      // Polaritt
    awd.uart_log_mode     = 0;      // Log-Mode 0
    awd.uart_app_mode     = 0;      // Append-Mode
    awd.stick_enable      = 0;      // USB-Stick-Enable (bisher nicht verwendet)
    awd.stick_dateiformat = 0;      // USB-Stick Dateiformat
    awd.stick_speed       = 0;      // USB-Stick Geschwindigkeit
    awd.lcd_hgbl          = 2;      // LCD-Hintergrundbeleuchtung
    awd.software_version  = 01;     // Firmware-Version
    awd.eeprom_code       = 0x55;   // Kennbyte fr Gltigkeit der Daten im EEPROM

    // Parameterdaten in EEPROM speichern
    eeprom_write_block((void*)&awd,&awd_ee,sizeof(awd));

    // Pause, nur fr Infozwecken
    _delay_ms(3000);
  }

  // UART initialisieren
  if (awd.uart01_pol) POL_RS232_H; else POL_RS232_L;    // PIN2/3 Polaritt
  usart_init(F_CPU);                                    // UART initialisieren

  // Datenspeicher lschen
  memset(&logbuch,0,sizeof(logbuch));
  logset.enable         = 0;
  logset.index          = 0;
  switch (awd.uart_log_mode) {
      case 0: logset.max = LOGBUCH0_MAX; break; // Mode 0
      case 1: logset.max = LOGBUCH1_MAX; break; // Mode 1
      case 2: logset.max = LOGBUCH2_MAX; break; // Mode 2
      case 3: logset.max = LOGBUCH3_MAX; break; // Mode 3
      case 4: logset.max = LOGBUCH4_MAX; break; // Mode 4
      case 5: logset.max = LOGBUCH5_MAX; break; // Mode 5
  }

  // Timer 0 initialisieren fr 1ms Wiederholrate
  TCCR0B = 0x05;    // Timer 0 1024-Vorteiler (55,5us/Takt bei 18.432000MHz)
  TIMSK0 = 0x01;    // Timer 0 Overflow  Interrupt freigeben

  // Generelle Interruptfreigabe des Systems
  sei();
}


// ********************************************************************
// Hauptprogramm
// ********************************************************************
int main (void) {
  init_system();                        // System initialisieren
  clear_lcd();                          // LCD lschen
  hgl_counter = HGL_ZEIT;               // Einschaltzeit der Hintergrundbeleuchtung
  if (awd.lcd_hgbl) LCD_HGBL_EIN;       // LCD Beleuchtung
  led_set(ledimage);                    // Schieberegister aktualisieren
  information(0);                       // Gerte-Infos anzeigen
  _delay_ms(2000);
  clear_lcd();

  // Hauptschleife
  while(1) {
    printf_lcd(0,0,"\x10UART Datenlogger\x11",1);
    sprintf(&lcd.buffer[0],"Log-Mode: %1d \xDD%s\xDD",awd.uart_log_mode,awd.uart_app_mode ? "APP" : "NEW");
    printf_lcd(0,1,lcd.buffer,1);
    sprintf(&lcd.buffer[0],"Timer: %lds",time_ms / 1000);
    printf_lcd(0,2,lcd.buffer,1);
    printf_lcd(0,3,"             <MEN\x5E>",1);
    if (((awd.lcd_hgbl == 2) && (!hgl_counter)) || (!awd.lcd_hgbl)) LCD_HGBL_AUS;         // Hintergrundbeleuchtung AUS
    temp_char = keypressed();   // Tastaturstatus
    if (temp_char) {
      // Taste gedrckt
      if (awd.lcd_hgbl) LCD_HGBL_EIN;   // Hintergrundbeleuchtung EIN
      led_set(ledimage);                // LEDs aktualisieren
      if (temp_char == ET) bedienung(); // Mensystem aufrufen
      if (awd.lcd_hgbl == 2) hgl_counter = HGL_ZEIT;   // Einschaltzeit der Hintergrundbeleuchtung neu setzen
      led_set(ledimage);                // LEDs aktualisieren
    }
    led_set(ledimage);                  // LEDs aktualisieren
  }
}

// ********************************************************************
// Timer 0 Interruptroutine
// Zeitinterval: 1ms
// ********************************************************************
ISR(TIMER0_OVF_vect) {
  // Berechnung: Wiederholintervall = (256 - TCNT0) * (1 / (FQuarz / Vorteiler))
  //             TCNT0              =  256 - (0.001 / (1 / (FQuarz / Vorteiler)))
  // Reloadwert = 238 (fr 1ms bei 18.432000MHz)
  TCNT0 = 238;

  // Timecode
  time_ms++;    // Time in [ms]
  if (time_reset) {
    // Time synchronisiert zurcksetzen
    time_ms     = 0;
    time_reset  = 0;
  }

  // Systemzhler
  if (taste_repeater)  taste_repeater--;    // Counter fr die Tastenwiederholung
  if (hgl_counter)     hgl_counter--;       // Counter fr die Hintergrundbeleuchtung
  if (timeout_counter) timeout_counter--;   // Counter fr Timeouts

  // Blinkfunktion (Sekundentakt)
  int_delay++;
  if (int_delay == 500) {
    blink = 1 - blink;
    int_delay = 0;
  }
  if (blink) LED_D1_EIN; else LED_D1_AUS;

  // RS-232 Statussignale
  if (RIN_RS232) LED_RI_AUS;  else LED_RI_EIN;
  if (DTR_RS232) LED_DTR_AUS; else LED_DTR_EIN;
  if (DSR_RS232) LED_DSR_AUS; else LED_DSR_EIN;
  if (RTS_RS232) LED_RTS_AUS; else LED_RTS_EIN;
  if (CTS_RS232) LED_CTS_AUS; else LED_CTS_EIN;
}
